/*****************************************************************************
 * $CAMITK_LICENCE_BEGIN$
 *
 * CamiTK - Computer Assisted Medical Intervention ToolKit
 * (c) 2001-2013 UJF-Grenoble 1, CNRS, TIMC-IMAG UMR 5525 (GMCAO)
 *
 * Visit http://camitk.imag.fr for more information
 *
 * This file is part of CamiTK.
 *
 * CamiTK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * CamiTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with CamiTK.  If not, see <http://www.gnu.org/licenses/>.
 *
 * $CAMITK_LICENCE_END$
 ****************************************************************************/

#include <QApplication>
#include <iostream>

#include "LevelWindow.h"
#include "ExtensionGenerator.h"
#include <Core.h>
using namespace camitk;

// CLI stuff
#include "CommandLineOptions.hxx"

// usage for this application
void usage(char *appName) {
    std::cerr << appName << std::endl;
    std::cerr << std::endl;
    std::cerr << "Usage: " << appName << " [options]" << std::endl;
    std::cerr << "Build using " << Core::version << std::endl;
    std::cerr << std::endl;
    std::cerr << "Options:" << endl;
    options::print_usage(std::cerr);
}

int main(int argc, char * argv[]) {
    try {
        QApplication app(argc, argv);
        int end; // End of options.
        options o(argc, argv, end);

        if (o.help()) {
            usage(argv[0]);
            return EXIT_SUCCESS;
        }
        else if(o.file() != "") {
            if (o.cepDir() == "") {
                usage(argv[0]);
                return EXIT_SUCCESS;
            }
            else {
                bool exitStatus = ExtensionGenerator::generateExtensionFiles(QString(o.file().c_str()), QString(o.cepDir().c_str()));

                if (!exitStatus) {
                    std::cerr << "Could not produce files correctly, sorry...\n" << std::endl;
                    return EXIT_FAILURE;
                }
            }
        }
        else if (o.skel()) {
            if (o.cepDir() == "") {
                usage(argv[0]);
                return EXIT_SUCCESS;
            }
            else {
                std::cerr << "Not Yet Implemented. Sorry...\n" << std::endl;
                return EXIT_FAILURE;
            }
        }
        else {
            // No file is given and this is not a skel generation either -> launch the GUI
            LevelWindow * wizard = new LevelWindow();
            wizard->show();
            return app.exec();
        }
    }
    catch (const cli::exception& e) {
        cerr << e << endl;
        usage(argv[0]);
        return EXIT_FAILURE;
    }
    catch (camitk::AbortException& e) {
        std::cout << argv[0] << " aborted..." << std::endl << "camitk AbortException:" << std::endl << e.what() << std::endl;
        return EXIT_FAILURE;
    }
    catch (std::exception& e) {
        std::cout << argv[0] << " aborted..." << std::endl << "std AbortException:" << std::endl << e.what() << std::endl;
        return EXIT_FAILURE;
    }
    catch (...) {
        std::cout << argv[0] << " aborted..." << std::endl << "Unknown Exception" << std::endl;
        return EXIT_FAILURE;
    }
}


